"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const koa_router_1 = __importDefault(require("koa-router"));
exports.GET_LIST = 'GET_LIST';
exports.GET_ONE = 'GET_ONE';
exports.CREATE = 'CREATE';
exports.UPDATE = 'UPDATE';
exports.DELETE = 'DELETE';
const ACTION_TO_FUNC = {
    [exports.GET_LIST]: getList,
    [exports.GET_ONE]: getOne,
    [exports.CREATE]: create,
    [exports.UPDATE]: update,
    [exports.DELETE]: delete_
};
function toJsonDefault(doc) {
    const o = doc.toObject();
    delete o._id;
    delete o.__v;
    return Object.assign({ id: doc._id }, o);
}
function rest(prefix, model, actions, toJson) {
    if (!actions) {
        actions = Object.keys(ACTION_TO_FUNC);
    }
    if (!toJson) {
        toJson = toJsonDefault;
    }
    const router = new koa_router_1.default({ prefix });
    actions.forEach(action => {
        ACTION_TO_FUNC[action](router, model, toJson);
    });
    return router;
}
function getList(router, model, toJson) {
    router.get('/', (ctx) => __awaiter(this, void 0, void 0, function* () {
        let { sort, range, filter } = ctx.query;
        if (sort) {
            const a = JSON.parse(sort);
            sort = { [a[0]]: a[1] === 'ASC' ? 1 : -1 };
        }
        let skip = 0;
        let limit = 100;
        if (range) {
            const a = JSON.parse(range);
            skip = a[0];
            limit = a[1] - skip;
        }
        filter = filter ? JSON.parse(filter) : {};
        if ('id' in filter) {
            filter._id = filter.id;
            delete filter.id;
        }
        const items = yield model
            .find(filter)
            .sort(sort)
            .skip(skip)
            .limit(limit)
            .exec();
        const total = yield model.count(filter);
        ctx.set('Content-Range', `${skip}-${skip + limit}/${total}`);
        ctx.body = yield Promise.all(items.map(item => toJson(item)));
    }));
}
function getOne(router, model, toJson) {
    router.get('/:id', (ctx, next) => __awaiter(this, void 0, void 0, function* () {
        const { id } = ctx.params;
        const item = yield model.findOne({ _id: id });
        if (!item) {
            ctx.throw(404, { error: 'Not fould' });
        }
        ctx.body = yield toJson(item);
    }));
}
function create(router, model, toJson) {
    router.post('/', (ctx, next) => __awaiter(this, void 0, void 0, function* () {
        const data = ctx.request.body;
        const item = yield model.create(data);
        ctx.status = 201;
        ctx.body = yield toJson(item);
    }));
}
function update(router, model, toJson) {
    router.put('/:id', (ctx, next) => __awaiter(this, void 0, void 0, function* () {
        const { id } = ctx.params;
        const data = ctx.request.body;
        console.log("data", data);
        const item = yield model.findOne({ _id: id });
        if (!item) {
            ctx.throw(404, { error: 'Not fould' });
        }
        item.set(data);
        yield item.save();
        ctx.body = yield toJson(item);
    }));
}
function delete_(router, model) {
    router.delete('/:id', (ctx, next) => __awaiter(this, void 0, void 0, function* () {
        const { id } = ctx.params;
        yield model.deleteOne({ _id: id });
        ctx.body = { id };
    }));
}
exports.default = rest;
